"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var traverse_object_1 = require("./traverse-object");
var utils_1 = require("./utils");
function functionMutatesInput(fn, args, thisArg) {
    if (thisArg === void 0) { thisArg = undefined; }
    var thisSnapshot = createSnapshot(thisArg);
    var argSnapshots = args.map(createSnapshot);
    fn.apply(thisArg, args);
    var thisChanges = checkSnapshotChanges(thisSnapshot);
    var argChanges = {};
    argSnapshots.forEach(function (snapshot, index) {
        var changes = checkSnapshotChanges(snapshot);
        if (changes.length > 0) {
            argChanges[index] = changes;
        }
    });
    if (thisChanges.length > 0 || utils_1.objectKeys(argChanges).length > 0) {
        return {
            args: argChanges,
            this: thisChanges
        };
    }
    else {
        return false;
    }
}
exports.functionMutatesInput = functionMutatesInput;
function createSnapshot(object) {
    var map = new Map();
    traverse_object_1.traverseObject(object, function (prop, path) {
        map.set(prop, { value: utils_1.objectAssign({}, prop), path: path });
    });
    return map;
}
function checkSnapshotChanges(snapshotMap) {
    var changes = [];
    snapshotMap.forEach(function (snapshot, object) {
        var newKeys = utils_1.objectKeys(object);
        var oldKeys = utils_1.objectKeys(snapshot.value);
        for (var _i = 0, _a = oldKeys.filter(function (k) { return newKeys.indexOf(k) < 0; }); _i < _a.length; _i++) {
            var key = _a[_i];
            changes.push({
                path: snapshot.path.concat([key]),
                oldValue: snapshot.value[key]
            });
        }
        for (var _b = 0, newKeys_1 = newKeys; _b < newKeys_1.length; _b++) {
            var key = newKeys_1[_b];
            var newValue = object[key];
            if (oldKeys.indexOf(key) < 0) {
                changes.push({
                    path: snapshot.path.concat([key]),
                    newValue: newValue
                });
            }
            else if (newValue !== snapshot.value[key]) {
                changes.push({
                    path: snapshot.path.concat([key]),
                    oldValue: snapshot.value[key],
                    newValue: newValue
                });
            }
        }
    });
    return changes;
}

//# sourceMappingURL=function-mutates-input.js.map
