"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
/**
 * Traverses an object and its child objects breadth-first and calls a callback for every object traversed.
 * Objects referenced more than once do not cause the callback to be called multiple times.
 */
function traverseObject(object, callback) {
    if (typeof object !== 'object' || !object)
        return;
    var traversed = new Set();
    var queue = [{ path: [], value: object }];
    do {
        var current = queue.splice(0, 1)[0];
        if (!traversed.has(current.value)) {
            traversed.add(current.value);
            callback(current.value, current.path);
            for (var _i = 0, _a = utils_1.objectKeys(current.value); _i < _a.length; _i++) {
                var key = _a[_i];
                var value = current.value[key];
                if (typeof value === 'object' && value !== null) {
                    queue.push({ path: current.path.concat([key]), value: value });
                }
            }
        }
    } while (queue.length > 0);
}
exports.traverseObject = traverseObject;

//# sourceMappingURL=traverse-object.js.map
