"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/** @internal */
function getFunctionName(fn, prototype) {
    if (exports.hasOwnProperty(fn, 'name') && fn.name) {
        return fn.name;
    }
    var source = Function.prototype.toString.apply(fn);
    var name = parseFunctionSource(source).name;
    if (name) {
        return name;
    }
    while (prototype) {
        for (var _i = 0, _a = Object.getOwnPropertyNames(prototype); _i < _a.length; _i++) {
            var name_1 = _a[_i];
            if (prototype[name_1] === fn) {
                return name_1;
            }
        }
        prototype = exports.objectGetPrototypeOf(prototype);
    }
    return '';
}
exports.getFunctionName = getFunctionName;
/** @internal */
function getFunctionArgs(fn) {
    var source = Function.prototype.toString.apply(fn);
    return parseFunctionSource(source).argNames;
}
exports.getFunctionArgs = getFunctionArgs;
/** @internal */
var functionRx = /^\s*(?:function\*? *([^)\n]*)\(([^)]*)\)|([^\s=\(,]+) *=>|\(([^\)]*)\) *=>|(\w+)\((?!function)([^)\n=<>]*?)\) *\{)/;
/**
 * Parses functions and arrows functions to get the function name and argument names.
 * @internal
 */
function parseFunctionSource(source) {
    var match = source.match(functionRx) || [];
    var functionName = match[1], functionArgs = match[2], arrowSingleArg = match[3], arrowArgs = match[4], methodName = match[5], methodArgs = match[6];
    var args = functionArgs || arrowSingleArg || arrowArgs || methodArgs || '';
    var name = functionName || methodName || '';
    var argNames = args
        .split(',')
        .filter(function (arg) { return arg.indexOf('...') < 0; })
        .map(function (arg) { return arg.replace(/^\s*(\w+)(?: *=.+)?\s*/, '$1'); })
        .filter(function (s) { return !!s; });
    return { name: name, argNames: argNames };
}
exports.parseFunctionSource = parseFunctionSource;
var classMetadataWeakMap;
var classMetadataSymbol = (typeof Symbol === 'function') ? Symbol('immutablets') : '@@immutablets';
var instanceMetadataWeakMap;
var instanceMetadataSymbol = (typeof Symbol === 'function') ? Symbol('immutablets-instance') : '@@immutablets-instance';
/** @internal */
function getImmutableClassMetadata(target) {
    if (typeof Reflect === 'object' && Reflect.getMetadata) {
        return Reflect.getMetadata('immutablets', target);
    }
    else if (typeof WeakMap === 'function') {
        if (!classMetadataWeakMap) {
            classMetadataWeakMap = new WeakMap();
        }
        do {
            var metadata = classMetadataWeakMap.get(target);
            if (metadata) {
                return metadata;
            }
            target = target.prototype && exports.objectGetPrototypeOf(target.prototype);
            target = target && target.constructor;
        } while (target);
    }
    else {
        do {
            var metadata = target[classMetadataSymbol];
            if (metadata) {
                return metadata;
            }
            target = target.prototype && exports.objectGetPrototypeOf(target.prototype);
            target = target && target.constructor;
        } while (target);
    }
}
exports.getImmutableClassMetadata = getImmutableClassMetadata;
/** @internal */
function setImmutableClassMetadata(target, data) {
    if (typeof Reflect === 'object' && Reflect.defineMetadata) {
        Reflect.defineMetadata('immutablets', data, target);
    }
    else if (typeof WeakMap === 'function') {
        if (!classMetadataWeakMap) {
            classMetadataWeakMap = new WeakMap();
        }
        classMetadataWeakMap.set(target, data);
    }
    else {
        Object.defineProperty(target, classMetadataSymbol, {
            configurable: true,
            enumerable: false,
            value: classMetadataSymbol,
            writable: false
        });
    }
}
exports.setImmutableClassMetadata = setImmutableClassMetadata;
/** @internal */
function getInstanceMetadata(target) {
    if (typeof WeakMap === 'function') {
        if (!instanceMetadataWeakMap) {
            instanceMetadataWeakMap = new WeakMap();
        }
        return instanceMetadataWeakMap.get(target);
    }
    else {
        return target[instanceMetadataSymbol];
    }
}
exports.getInstanceMetadata = getInstanceMetadata;
/** @internal */
function setInitialInstanceMetadata(target) {
    if (typeof WeakMap === 'function') {
        if (!instanceMetadataWeakMap) {
            instanceMetadataWeakMap = new WeakMap();
        }
        instanceMetadataWeakMap.set(target, {
            callDepth: 0,
            observers: []
        });
    }
    else {
        Object.defineProperty(target, instanceMetadataSymbol, {
            configurable: true,
            enumerable: false,
            value: {
                callDepth: 0,
                changeObservers: []
            },
            writable: false
        });
    }
}
exports.setInitialInstanceMetadata = setInitialInstanceMetadata;
/** @internal Short-hand method for Array.prototype.slice(). */
exports.arraySlice = Function.prototype.call.bind(Array.prototype.slice);
/** @internal Short-hand method for Object.hasOwnProperty(). */
exports.hasOwnProperty = Function.call.bind(Object.prototype.hasOwnProperty);
/** @internal Short-hand method for Array.isArray(). */
exports.isArray = Array.isArray;
/** @internal Short-hand method for Object.keys(). */
exports.objectKeys = Object.keys;
/** @internal Short-hand method for Object.assign(). */
exports.objectAssign = Object.assign;
/** @internal Short-hand method for Object.assign(). */
exports.objectCreate = Object.create;
/** @internal Short-hand method for Object.defineProperty(). */
exports.objectDefineProperty = Object.defineProperty;
/** @internal Short-hand method for Object.getPrototypeOf(). */
exports.objectGetPrototypeOf = Object.getPrototypeOf;

//# sourceMappingURL=utils.js.map
